#pragma once

#include "TM.h"
#include "TimeFunctions.h"
#include "json_functions.h"

static bool loadjson(const jsonval& parent, const std::string& key, std::vector<TimeJD>& vec, const bool exist = true, bool* loaded = nullptr)
{
    jsonval::ConstMemberIterator itr;
    if (!extract_value(parent, key, exist, loaded, itr)) {
        return false;
    }

    const jsonval& a = itr->value;

    if (!a.IsArray()) {
        throw Exp() << "error: element is not an array";
    }

    for (jsonval::ConstValueIterator it = a.Begin(); it != a.End(); ++it) {
        if (it->IsString()) {
            vec.emplace_back(StrToDate(it->GetString()));
        }
        else {
            throw Exp() << "error: unsupported type";
        }
    }

    return true;
}

static void loadjson(const jsonval& parent, const std::string& key, TimeJD& value, bool exist = true, bool* loaded = nullptr)
{
    const jsonval::ConstMemberIterator itr = parent.FindMember(key.c_str());

    if (itr == parent.MemberEnd() && exist) {
        throw Exp() << "error: element not found " << key;
    }
    else if (itr == parent.MemberEnd() && !exist) {
        if (loaded) {
            *loaded = false;
        }
        return;
    }

    const jsonval& a = itr->value;

    if (a.IsString()) {
        TimeGD gd;
        StrToDate(a.GetString(), gd);
        value = gd;
        if (loaded) {
            *loaded = true;
        }
    }
    else {
        throw (Exp() << "element '" << key << "'" << " has unexpected format '");
    }
}

static void loadjson(const jsonval& parent, const std::string& key, TimeGD& value, bool exist = true)
{
    const jsonval::ConstMemberIterator itr = parent.FindMember(key.c_str());

    if (itr == parent.MemberEnd() && exist) {
        throw Exp() << "error: element not found";
    }

    const jsonval& a = itr->value;

    if (a.IsString()) {
        StrToDate(a.GetString(), value);
    }
    else {
        throw (Exp() << "element '" << key << "'" << " has unexpected format '");
    }
}

static void PutValue(jsonalloc& alc, jsonval& ss, const std::string& conName, const TimeGD& value)
{
    jsonval jname(conName, alc);
    jsonval jvalue(DateToStr(value), alc);
    ss.AddMember(jname, jvalue, alc);
}

static void PutValue(jsonalloc& alc, jsonval& ss, const std::string& conName, const TimeJD& value)
{
    jsonval jname(conName, alc);
    jsonval jvalue(DateToStr(value), alc);
    ss.AddMember(jname, jvalue, alc);
}


static bool loadjson(const jsonval& parent, const std::string& key, svalue<TimeJD>& value, const bool exist = true)
{
    jsonval::ConstMemberIterator itr;

    bool* loaded = nullptr;
    if (!extract_value(parent, key, exist, loaded, itr)) {
        return false;
    }

    const jsonval& a = itr->value;

    if (a.IsString()) {
        value = StrToDate(a.GetString());
    }
    else {
        throw (Exp() << "element '" << key << "'" << " has unexpected format '");
    }

    return true;
}

static bool loadjson(const std::map<std::string, std::string>& parent, const std::string& key, svalue<TimeJD>& value, const bool exist = true)
{
    std::map<std::string, std::string>::const_iterator itr;

    bool* loaded = nullptr;
    if (!extract_value(parent, key, exist, loaded, itr)) {
        return false;
    }

    value = StrToDate(itr->second);

    return true;
}

static bool loadjson(const std::map<std::string, std::string>& parent, const std::string& key, TimeJD& value, const bool exist = true, bool* loaded = nullptr)
{
    std::map<std::string, std::string>::const_iterator itr;

    if (!extract_value(parent, key, exist, loaded, itr)) {
        return false;
    }

    const std::string& str = itr->second;

    if (str.empty() && !exist) {
        if (loaded) {
            *loaded = false;
        }
        return false;
    }

    value = StrToDate(str);

    return true;
}
